open Glfw
open Gl
module B = Bigarray

let test_glerror s =
	let e = glGetError () in
	let se () =
		if e = cgl_no_error then "No Error"
		else if e = cgl_invalid_value then "Invalid Value"
		else if e = cgl_invalid_enum then "Invalid Enum"
		else if e = cgl_invalid_operation then "Invalid Operation"
		else if e = cgl_stack_overflow then "Stack Overflow"
		else if e = cgl_stack_underflow then "Stack Underflow"
		else if e = cgl_out_of_memory then "Out of Memory"
		else "Unknown Error: "^(string_of_int e)
	in
	if e <> cgl_no_error then begin
		Printf.eprintf "GL_ERROR (%s): %s\n" s (se ()); 
		flush stdout;
	end


let pi = 3.14159

let a = B.Array1.create B.float64 B.c_layout 16 
let look_at ex ey ez cx cy cz ux uy uz =
	let z0 = ex -. cx and z1 = ey -. cy and z2 = ez -. cz in
	let mag = sqrt (z0 *. z0 +. z1 *. z1 +. z2 *. z2) in
	let z0 = if mag = 0.0 then z0 else z0 /. mag in
	let z1 = if mag = 0.0 then z1 else z1 /. mag in
	let z2 = if mag = 0.0 then z2 else z2 /. mag in
	let x0 = uy *. z2 -. uz *. z1 in
	let x1 = (~-. ux) *. z2 +. uz *. z0 in
	let x2 = ux *. z1 -. uy *. z0 in
	let y0 = z1 *. x2 -. z2 *. x1 in
	let y1 = z2 *. x0 -. z0 *. x2 in
	let y2 = z0 *. x1 -. z1 *. x0 in
	let mag = sqrt (y0 *. y0 +. y1 *. y1 +. y2 *. y2) in
	let y0 = if mag = 0.0 then y0 else y0 /. mag in
	let y1 = if mag = 0.0 then y1 else y1 /. mag in
	let y2 = if mag = 0.0 then y2 else y2 /. mag in
	let mag = sqrt (x0 *. x0 +. x1 *. x1 +. x2 *. x2) in
	let x0 = if mag = 0.0 then x0 else x0 /. mag in
	let x1 = if mag = 0.0 then x1 else x1 /. mag in
	let x2 = if mag = 0.0 then x2 else x2 /. mag in
	let f row col v = a.{col*4+row} <- v in
	f 0 0 x0; f 0 1 x1; f 0 2 x2; f 0 3 0.0;
	f 1 0 y0; f 1 1 y1; f 1 2 y2; f 1 3 0.0;
	f 2 0 z0; f 2 1 z1; f 2 2 z2; f 2 3 0.0;
	f 3 0 0.0; f 3 1 0.0; f 3 2 0.0; f 3 3 1.0;
	glMultMatrixd a;
	glTranslated (-.ex) (-.ey) (-.ez)

let perspective fovy aspect znear zfar = 
    let ymax = znear *. tan (fovy *. pi /. 360.0) in
    let ymin = ~-. ymax in
    let xmin = ymin *. aspect in
    let xmax = ymax *. aspect in
    glFrustum xmin xmax ymin ymax znear zfar
    
let _ =
    try
	if not (init ()) then failwith "Glfw.init failed";
	if not (open_window 640 480 0 0 0 0 0 0 WINDOW) then failwith "Glfw.open_window failed";
	enable STICKY_KEYS;
	swap_interval 0;
	let tt = Unix.gettimeofday () in
	let rec g f t0 =
	    let t = Unix.gettimeofday () in
	    let x,y = get_mouse_pos () in
	    let f,t0 =
		if t -. t0 > 1.0 || f = 0 then begin
		    let fps = float f /. (t -. t0) in
		    let s = Printf.sprintf "Spinning triangle (%.1f FPS)" fps in
		    set_window_title s;
		    1,t
		end
		else f+1,t0
	    in
	    let width,height = get_window_size () in
	    let height = if height > 0 then height else 1 in
	    glViewport 0 0 width height;
	    glClearColor 0.0 0.0 0.0 0.0;
	    glClear cgl_color_buffer_bit;
	    glMatrixMode cgl_projection;
	    glLoadIdentity ();
	    perspective 65.0 (float width /. float height) 1.0 100.0;
	    glMatrixMode cgl_modelview;
	    glLoadIdentity ();
	    look_at 0.0 1.0 0.0 0.0 20.0 0.0 0.0 0.0 1.0;
	    glTranslatef 0.0 14.0 0.0;
	    glRotatef (0.3 *. float x +. (t -. tt) *. 100.0) 0.0 0.0 1.0;
	    glBegin cgl_triangles;
		glColor3f 1.0 0.0 0.0;
		glVertex3f (-5.0) 0.0 (-4.0);
		glColor3f 0.0 1.0 0.0;
		glVertex3f 5.0 0.0 (-4.0);
		glColor3f 0.0 0.0 1.0;
		glVertex3f 0.0 0.0 6.0;
	    glEnd ();
	    test_glerror "main";
	    swap_buffers ();
	    if get_key key_esc || get_window_param OPENED = 0 then () else g f t0
	in
	g 0 tt;
	terminate ()
    with Failure s -> Printf.eprintf "Failure: %s\n" s
