open Gl

let draw () =
	glClear cgl_color_buffer_bit;
	glClear cgl_depth_buffer_bit;
	
	(* your code *)
	
	Hgl.test_glerror "draw";
	Sdl.gl_swap_buffers ()

let reshape x y =
	glViewport 0 0 x y;
	glMatrixMode cgl_projection;
	glLoadIdentity ();
	glMatrixMode cgl_modelview;
	Hgl.test_glerror "reshape";
	()

let r_fpsshow = ref false
let rec mainloop surf fps t0 =
	let quit () = Sdl.free_surface surf; Sdl.quit (); exit 0 in
	let _ = (* events match *)
		match Sdl.poll_event () with
		| Sdl.Equit -> quit ()
		| Sdl.Resize (x,y) -> reshape x y
		| Sdl.Key (_,s,_,k,_,_) -> if s = Sdl.RELEASED then begin 
			match k with
			| Sdl.K_ESCAPE -> quit ()
			| _ -> ()
			end
		| _ -> ()
	in
	draw ();
	(* fps'ing *)
	let t = Sdl.get_ticks () in
	let dt = t - t0 in
	if dt < 1000 then mainloop surf (fps+1) t0 
	else begin
		if !r_fpsshow then begin
		    print_float ((float fps)/.(float dt) *. 1000.0); 
		    print_newline ();
		end;
		mainloop surf 1 t;
	end

let init () = (* GL init *)
	Hgl.test_glerror "init";
	()

let _ =
	let width = ref 640 in
	let height = ref 480 in
	let bpp = ref 0 in
	let fullscreen = ref false in
	let flags = [Sdl.OPENGL] in
	let init_gl () = 
		reshape !width !height;
		init ();
		()
	in
	let _ = (* argument analyze *)
	    let rec keywords =
	    [
			"-w", Arg.Int (fun i -> width := i), "set width";
			"-h", Arg.Int (fun i -> height := i), "set height";
			"-bpp", Arg.Int (fun i -> bpp := i), "set bpp";
			"-fps", Arg.Set r_fpsshow, "show fps";
			"-f", Arg.Set fullscreen, "toggle fullscreen mode";
			"-help", Arg.Unit usage, "get help";
			"--help", Arg.Unit usage, "get help";
	    ]
	    and usage () =
			Arg.usage keywords "";
			exit 0;
	    in
	    Arg.parse keywords (fun _ -> usage ()) ""
	in
	Sdl.init [Sdl.INIT_VIDEO];
	let f = flags @ if !fullscreen then [Sdl.FULLSCREEN] else [] in
	let surf = Sdl.set_video_mode !width !height !bpp f in
	init_gl ();
	mainloop surf 0 (Sdl.get_ticks ())
