open Gl

type t =
{
    mutable tx:float;
    mutable ty:float;
    mutable ts:float;
    mutable tc:float;
}

let r_speed = ref 1.0
let a = Array.init 1000 (fun _ -> 
    { 
	tx=Random.float 2.0 -. 1.0; 
	ty=Random.float 2.0 -. 1.0; 
	ts=Random.float 0.001; 
	tc=Random.float 0.4 +. 0.6;
    }
)

let init_array () = 
    let mcompare a b = ~- (compare a b) in
    let cmp t1 t2 = compare t1.tc t2.tc in
    Array.sort cmp a

let cycle a = 
    if a > 0.0 then (mod_float (a +. 1.0) 2.0) -. 1.0 
    else (mod_float (a -. 1.0) (-2.0)) +. 1.0 
    
let r_animate = ref false
let draw () =
	glClear (cgl_color_buffer_bit lor cgl_depth_buffer_bit);
	
	(* your code *)
	glBegin cgl_points;
	Array.iter (fun t ->
	    let c = t.ts *. 1000.0 in
	    glColor3f c c c;
	    glVertex2f t.tx t.ty;
	    if !r_animate then t.tx <- cycle (t.tx +. t.ts *. !r_speed);
	) a;
	glEnd ();
	
	Glfw.swap_buffers ()

let reshape x y =
	glViewport 0 0 x y;
	glMatrixMode cgl_projection;
	glLoadIdentity ();
	glMatrixMode cgl_modelview;
	()

let init () = (* GL init *)
	glEnable cgl_blend;
	glBlendFunc cgl_src_alpha cgl_one_minus_src_alpha;
	glEnable cgl_point_smooth;
	glDisable cgl_depth_test;
	glPointSize 4.0;
	glHint cgl_point_smooth_hint cgl_nicest;
	init_array ();
	()

let key a b = if b then match a with
| _ when a = Glfw.key_esc -> raise Exit
| _ when a = Glfw.key_space -> r_animate := not !r_animate
| _ when a = Glfw.key_up -> r_speed := !r_speed +. 0.1
| _ when a = Glfw.key_down -> r_speed := !r_speed -. 0.1
| _ -> ()

let _ =
    try
	if not (Glfw.init ()) then failwith "Glfw.init failed";
	if not (Glfw.open_window 800 600 0 0 0 8 24 0 Glfw.WINDOW) then failwith "Glfw.open_window failed";
	Glfw.enable Glfw.STICKY_KEYS;
	Glfw.swap_interval 1;
	init ();
	Glfw.set_key_cb key;
	reshape 800 600;
	while true do
	    draw ();
    	    if Glfw.get_window_param Glfw.OPENED = 0 then raise Exit;
	done
    with
    | Failure s -> Printf.eprintf "Failure: %s\n" s
    | Exit -> Glfw.terminate ()

