open Gl

let draw () = 
    glClear (cgl_color_buffer_bit lor cgl_depth_buffer_bit);
    glDrawArrays cgl_quads 0 4;
    Glfw.swap_buffers ()

let reshape x y =
    glViewport 0 0 x y;
    glMatrixMode cgl_projection;
    glLoadIdentity ();
    glMatrixMode cgl_modelview;
    glLoadIdentity ()

let key k b = if b then match k with
| _ when k = Glfw.key_esc -> raise Exit
| _ -> ()

let _ =
    try
	if not (Glfw.init ()) then failwith "Glfw.init failed";
	if not (Glfw.open_window 800 600 0 0 0 8 24 8 Glfw.WINDOW) then failwith "open_window failed";
	let nvsize = 12*4+4*4 in
	let nv = _glAllocateMemoryNV nvsize 0.0 0.0 1.0 in
	if Hgl.ba_size nv = 0 then failwith "allocate memory failed";
	let ca = Hgl.ba_ofs_byte nv 0 16 in
	let va = Hgl.ba_ofs_float nv 16 12 in
	for i = 0 to 3 do 
	    va.{i*3+0} <- if i<2 then 0.5 else (-0.5);
	    va.{i*3+1} <- if i=0 || i=3 then (-0.5) else (0.5);
	    va.{i*3+2} <- 0.0;
	    ca.{i*4+0} <- 255;
	    ca.{i*4+1} <- 255;
	    ca.{i*4+2} <- 255;
	    ca.{i*4+3} <- 255;
	done;
	glVertexPointer 3 cgl_float 0 va;
	glColorPointer 4 cgl_unsigned_byte 0 ca;
	glEnableClientState cgl_vertex_array;
	glEnableClientState cgl_color_array;
	glVertexArrayRangeNV nvsize nv;
	glEnableClientState cgl_vertex_array_range_nv;
	Printf.printf "VAR state: %b\n" (Hgl.get_boolean cgl_vertex_array_range_valid_nv);
	flush stdout;
	Glfw.set_window_size_cb reshape;
	Glfw.set_key_cb key;
	while true do 
	    draw ();
	    if Glfw.get_window_param Glfw.OPENED = 0 then raise Exit;
	done
    with 
    | Failure s -> Printf.eprintf "Failure: %s\n" s
    | Exit -> Glfw.terminate ()
